<?php
namespace Geral\Logic\ObrigacoesLegais\Builder;

class DescricaoDoArquivo
{


    private $arquivo;
    private $versao;
    private $elementos;

    private $raiz;

    private $builtObj;

    const ATRIBUTO = "A";
    const ELEMENTO = "E";
    const GRUPO    = "G";

    /**
     * Metodo construtor.
     *
     * @param $arquivo   String Nome do arquivo especificado do arquivo de integracao (ex.: S-1100, para o e-Social).
     * @param $versao    String Versão do arquivo, em relação ao manual onde foi especificado (ex.: 1.1).
     * @param $elementos array  Array com os dados dos elementos que compõe o arquivo. Cada elemento deste array deverá
     *                          apresentar a seguinte estrutura:
     *                              "CAMPO"      : "<nome_do_campo>",
     *                              "PAI"        : "<nome_do_elemento_pai>",
     *                              "ELEMENTO"   : "<G,E,A (grupo, elemento ou atributo)>",
     *                              "TIPO"       : "<null, D, C, N (char, number, date)>",
     *                              "OCORRENCIA" : "<ocorrencia_do_elemento> (ex.: 1-1)",
     *                              "DESCRICAO"  : "<descricao_do_elemento>",
     *                              "CONDICAO"   : "<condicao_de_aceitacao_do_elemento>",
     * @param $raiz      String Especifica o nome do elemento raiz do arquivo. Caso nada seja passado, o builder deverá
     *                          pesquisar dentre os elementos aquele que nao possui o atributo "pai" e assumir este como 
     *                          a raiz do arquivo.
     *
     * @return void
     */
    public function __construct($arquivo, $versao, $elementos = array(), $raiz = null) 
    {
        $this->arquivo   = $arquivo;
        $this->versao    = $versao;
        $this->elementos = $elementos;
        $this->raiz      = $raiz;
    }


    public function buildData() 
    {
        $dataCriacaoArquivo = new \DateTime();

        $this->builtObj["__raiz"]     = $this->raiz ? : $this->procurarElementoRaiz();
        $this->builtObj["__versao"]   = $this->versao;
        $this->builtObj["__arquivo"]  = $this->arquivo;
        $this->builtObj["__dataHora"] = $dataCriacaoArquivo->format("d/m/Y H:i:s");
        $this->buildComponent();
        $this->setElementosFilhos();
        return $this->builtObj;
    }

    private function setElementosFilhos() 
    {
        foreach ($this->elementos as $elemento) {
            if ($elemento["PAI"]) {
                if ($elemento["ELEMENTO"] === self::ATRIBUTO) {
                    if (!in_array($elemento["CAMPO"], $this->builtObj[$elemento["PAI"]]["atributos"])) {
                        $this->builtObj[$elemento["PAI"]]["atributos"][] = $elemento["CAMPO"];
                    }
                } else {
                    if (!in_array($elemento["CAMPO"], $this->builtObj[$elemento["PAI"]]["filhos"])) {
                        $this->builtObj[$elemento["PAI"]]["filhos"][] = $elemento["CAMPO"];
                    }
                }
            }
        }
    }

    private function buildComponent() 
    {
        foreach ($this->elementos as $elemento) {
            $this->builtObj[$elemento["CAMPO"]] = array (
                "nomeElemento" => $elemento["CAMPO"],
                "ocorrencia"   => $this->getOcorrencias($elemento["OCORRENCIA"]),
                "condicao"     => $elemento["CONDICAO"],
                "tipo"         => $elemento["ELEMENTO"],
                "validacoes"   => $this->getValidacoes($elemento["DESCRICAO"]),
                "atributos"    => array(),
                "filhos"       => array(),
            );
        }
    }

    private function getOcorrencias($str) 
    {
        list($menor, $maior) = explode("-", $str);
        return array(
            "menor" => $menor,
            "maior" => $maior,
        );
    }

    private function getValidacoes($descricao) 
    {
        preg_match_all("/REGRA_\w+/", $descricao, $result);
        return $result[0];
    }

    private function procurarElementoRaiz() 
    {
        foreach ($this->elementos as $elemento) {
            if ((bool)$elemento["PAI"] === false) {
                return $elemento["CAMPO"];
            }
        }

        return null;
    }

}